use crate::samplic::{
    Curve, Measurement, MeasurementMetadata, MeasurementResults,
    ProfileDescription, Sample, SampleMetadata,
};
use crate::Result;
use indexmap::IndexSet;
use uuid::Uuid;

pub trait StorableDevice {
    fn store_user_profile(
        &mut self,
        profile_id: Uuid,
        profile: &ProfileDescription,
    ) -> Result<()>;

    fn store_measurement_metadata(
        &mut self,
        sample_id: Uuid,
        measurement_index: usize,
        metadata: &MeasurementMetadata,
    ) -> Result<()>;

    fn store_results(
        &mut self,
        sample_id: Uuid,
        measurement_index: usize,
        results: &MeasurementResults,
    ) -> Result<()>;

    fn store_curve(
        &mut self,
        sample_id: Uuid,
        measurement_index: usize,
        curve_id: &str,
        curve: &Curve,
    ) -> Result<()>;

    fn store_measurement(
        &mut self,
        sample_id: Uuid,
        measurement_index: usize,
        measurement: &Measurement,
    ) -> Result<()>;

    fn store_sample_metadata(
        &mut self,
        sample_id: Uuid,
        metadata: &SampleMetadata,
    ) -> Result<()>;

    fn store_sample(
        &mut self,
        sample_id: Uuid,
        sample: &Sample,
    ) -> Result<()>;

    fn store_sample_index(&mut self, index: &IndexSet<Uuid>)
        -> Result<()>;
}
